@@ -2,9 +2,12 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) The Exim Maintainers 2022 */
+/* Copyright (c) University of Cambridge 1995 - 2021 */
 /* See the file NOTICE for conditions of use and distribution. */
 
+#ifndef DBFUNCTIONS_H
+#define DBFUNCTIONS_H
 
 /* Functions for reading/writing exim database files */
 
@@ -12,6 +15,7 @@ void     dbfn_close(open_db *);
 int      dbfn_delete(open_db *, const uschar *);
 open_db *dbfn_open(uschar *, int, open_db *, BOOL, BOOL);
 void    *dbfn_read_with_length(open_db *, const uschar *, int *);
+void    *dbfn_read_enforce_length(open_db *, const uschar *, size_t);
 uschar  *dbfn_scan(open_db *, BOOL, EXIM_CURSOR **);
 int      dbfn_write(open_db *, const uschar *, void *, int);
 
@@ -23,11 +27,12 @@ int      dbfn_write(open_db *, const uschar *, void *, int);
 changed at release 4.3. */
 
 #if defined(USE_DB) && defined(DB_VERSION_STRING)
-#if DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
+# if DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
 void     dbfn_bdb_error_callback(const DB_ENV *, const char *, const char *);
-#else
+# else
 void     dbfn_bdb_error_callback(const char *, char *);
-#endif
+# endif
 #endif
 
+#endif
 /* End of dbfunctions.h */
