@@ -2,8 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 /* DMARC support.
+   Copyright (c) The Exim Maintainers 2019 - 2022
    Copyright (c) Todd Lyons <tlyons@exim.org> 2012 - 2014
-   Copyright (c) The Exim Maintainers 2019
    License: GPL */
 
 /* Portions Copyright (c) 2012, 2013, The Trusted Domain Project;
@@ -51,21 +51,32 @@ static dmarc_exim_p dmarc_policy_description[] = {
   { US"reject",     DMARC_RECORD_P_REJECT },
   { NULL,           0 }
 };
+
+
+gstring *
+dmarc_version_report(gstring * g)
+{
+return string_fmt_append(g, "Library version: dmarc: Compile: %d.%d.%d.%d\n",
+    (OPENDMARC_LIB_VERSION & 0xff000000) >> 24, (OPENDMARC_LIB_VERSION & 0x00ff0000) >> 16,
+    (OPENDMARC_LIB_VERSION & 0x0000ff00) >> 8, OPENDMARC_LIB_VERSION & 0x000000ff);
+}
+
+
 /* Accept an error_block struct, initialize if empty, parse to the
- * end, and append the two strings passed to it.  Used for adding
- * variable amounts of value:pair data to the forensic emails. */
+end, and append the two strings passed to it.  Used for adding
+variable amounts of value:pair data to the forensic emails. */
 
 static error_block *
 add_to_eblock(error_block *eblock, uschar *t1, uschar *t2)
 {
 error_block *eb = store_malloc(sizeof(error_block));
-if (eblock == NULL)
+if (!eblock)
   eblock = eb;
 else
   {
   /* Find the end of the eblock struct and point it at eb */
   error_block *tmp = eblock;
-  while(tmp->next != NULL)
+  while(tmp->next)
     tmp = tmp->next;
   tmp->next = eb;
   }
@@ -76,8 +87,8 @@ return eblock;
 }
 
 /* dmarc_init sets up a context that can be re-used for several
-   messages on the same SMTP connection (that come from the
-   same host with the same HELO string) */
+messages on the same SMTP connection (that come from the
+same host with the same HELO string) */
 
 int
 dmarc_init()
@@ -218,7 +229,11 @@ if (rc == DNS_SUCCEED)
   for (dns_record * rr = dns_next_rr(dnsa, &dnss, RESET_ANSWERS); rr;
        rr = dns_next_rr(dnsa, &dnss, RESET_NEXT))
     if (rr->type == T_TXT && rr->size > 3)
-      return string_copyn(US rr->data, rr->size);
+      {
+      store_free_dns_answer(dnsa);
+      return string_copyn_taint(US rr->data, rr->size, GET_TAINTED);
+      }
+store_free_dns_answer(dnsa);
 return NULL;
 }
 
@@ -226,6 +241,8 @@ return NULL;
 static int
 dmarc_write_history_file()
 {
+int history_file_fd;
+ssize_t written_len;
 int tmp_ans;
 u_char **rua; /* aggregate report addressees */
 uschar *history_buffer = NULL;
@@ -235,6 +252,14 @@ if (!dmarc_history_file)
   DEBUG(D_receive) debug_printf("DMARC history file not set\n");
   return DMARC_HIST_DISABLED;
   }
+history_file_fd = log_open_as_exim(dmarc_history_file);
+
+if (history_file_fd < 0)
+  {
+  log_write(0, LOG_MAIN|LOG_PANIC, "failure to create DMARC history file: %s",
+			   dmarc_history_file);
+  return DMARC_HIST_FILE_ERR;
+  }
 
 /* Generate the contents of the history file */
 history_buffer = string_sprintf(
@@ -286,28 +311,16 @@ if (host_checking || f.running_in_test_harness)
   }
 else
   {
-  ssize_t written_len;
-  const int history_file_fd = log_open_as_exim(dmarc_history_file);
-
-  if (history_file_fd < 0)
-    {
-    log_write(0, LOG_MAIN|LOG_PANIC, "failure to create DMARC history file: %s",
-			   dmarc_history_file);
-    return DMARC_HIST_FILE_ERR;
-    }
-
   written_len = write_to_fd_buf(history_file_fd,
 				history_buffer,
 				Ustrlen(history_buffer));
-
-  (void)close(history_file_fd);
-
-  if (written_len <= 0)
+  if (written_len == 0)
     {
     log_write(0, LOG_MAIN|LOG_PANIC, "failure to write to DMARC history file: %s",
 			   dmarc_history_file);
     return DMARC_HIST_WRITE_ERR;
     }
+  (void)close(history_file_fd);
   }
 return DMARC_HIST_OK;
 }
@@ -517,7 +530,7 @@ if (!dmarc_abort && !sender_host_authenticated)
   /* Can't use exim's string manipulation functions so allocate memory
   for libopendmarc using its max hostname length definition. */
 
-  dmarc_domain = store_get(DMARC_MAXHOSTNAMELEN, TRUE);
+  dmarc_domain = store_get(DMARC_MAXHOSTNAMELEN, GET_TAINTED);
   libdm_status = opendmarc_policy_fetch_utilized_domain(dmarc_pctx,
     dmarc_domain, DMARC_MAXHOSTNAMELEN-1);
   store_release_above(dmarc_domain + Ustrlen(dmarc_domain)+1);
