@@ -2,8 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2020 - 2022 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
-/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for writing log files. The code for maintaining datestamped
@@ -12,7 +12,6 @@ log files was originally contributed by Tony Sheen. */
 
 #include "exim.h"
 
-#define LOG_NAME_SIZE 256
 #define MAX_SYSLOG_LEN 870
 
 #define LOG_MODE_FILE   1
@@ -30,7 +29,6 @@ static uschar *log_names[] = { US"main", US"reject", US"panic", US"debug" };
 
 static uschar mainlog_name[LOG_NAME_SIZE];
 static uschar rejectlog_name[LOG_NAME_SIZE];
-static uschar debuglog_name[LOG_NAME_SIZE];
 
 static uschar *mainlog_datestamp = NULL;
 static uschar *rejectlog_datestamp = NULL;
@@ -55,66 +53,68 @@ static size_t pid_position[2];
 number definitions in macros.h */
 
 static const uschar * exim_errstrings[] = {
-  US"",
-  US"unknown error",
-  US"user slash",
-  US"exist race",
-  US"not regular",
-  US"not directory",
-  US"bad ugid",
-  US"bad mode",
-  US"inode changed",
-  US"lock failed",
-  US"bad address2",
-  US"forbid pipe",
-  US"forbid file",
-  US"forbid reply",
-  US"missing pipe",
-  US"missing file",
-  US"missing reply",
-  US"bad redirect",
-  US"smtp closed",
-  US"smtp format",
-  US"spool format",
-  US"not absolute",
-  US"Exim-imposed quota",
-  US"held",
-  US"Delivery filter process failure",
-  US"Delivery add/remove header failure",
-  US"Delivery write incomplete error",
-  US"Some expansion failed",
-  US"Failed to get gid",
-  US"Failed to get uid",
-  US"Unset or non-existent transport",
-  US"MBX length mismatch",
-  US"Lookup failed routing or in smtp tpt",
-  US"Can't match format in appendfile",
-  US"Creation outside home in appendfile",
-  US"Can't check a list; lookup defer",
-  US"DNS lookup defer",
-  US"Failed to start TLS session",
-  US"Mandatory TLS session not started",
-  US"Failed to chown a file",
-  US"Failed to create a pipe",
-  US"When verifying",
-  US"When required by client",
-  US"Used internally in smtp transport",
-  US"RCPT gave 4xx error",
-  US"MAIL gave 4xx error",
-  US"DATA gave 4xx error",
-  US"Negotiation failed for proxy configured host",
-  US"Authenticator 'other' failure",
-  US"target not supporting SMTPUTF8",
-  US"host is local",
-  US"tainted filename",
+  [0] = US"",
+  [- ERRNO_UNKNOWNERROR] =	US"unknown error",
+  [- ERRNO_USERSLASH] =		US"user slash",
+  [- ERRNO_EXISTRACE] =		US"exist race",
+  [- ERRNO_NOTREGULAR] =	US"not regular",
+  [- ERRNO_NOTDIRECTORY] =	US"not directory",
+  [- ERRNO_BADUGID] =		US"bad ugid",
+  [- ERRNO_BADMODE] =		US"bad mode",
+  [- ERRNO_INODECHANGED] =	US"inode changed",
+  [- ERRNO_LOCKFAILED] =	US"lock failed",
+  [- ERRNO_BADADDRESS2] =	US"bad address2",
+  [- ERRNO_FORBIDPIPE] =	US"forbid pipe",
+  [- ERRNO_FORBIDFILE] =	US"forbid file",
+  [- ERRNO_FORBIDREPLY] =	US"forbid reply",
+  [- ERRNO_MISSINGPIPE] =	US"missing pipe",
+  [- ERRNO_MISSINGFILE] =	US"missing file",
+  [- ERRNO_MISSINGREPLY] =	US"missing reply",
+  [- ERRNO_BADREDIRECT] =	US"bad redirect",
+  [- ERRNO_SMTPCLOSED] =	US"smtp closed",
+  [- ERRNO_SMTPFORMAT] =	US"smtp format",
+  [- ERRNO_SPOOLFORMAT] =	US"spool format",
+  [- ERRNO_NOTABSOLUTE] =	US"not absolute",
+  [- ERRNO_EXIMQUOTA] =		US"Exim-imposed quota",
+  [- ERRNO_HELD] =		US"held",
+  [- ERRNO_FILTER_FAIL] =	US"Delivery filter process failure",
+  [- ERRNO_CHHEADER_FAIL] =	US"Delivery add/remove header failure",
+  [- ERRNO_WRITEINCOMPLETE] =	US"Delivery write incomplete error",
+  [- ERRNO_EXPANDFAIL] =	US"Some expansion failed",
+  [- ERRNO_GIDFAIL] =		US"Failed to get gid",
+  [- ERRNO_UIDFAIL] =		US"Failed to get uid",
+  [- ERRNO_BADTRANSPORT] =	US"Unset or non-existent transport",
+  [- ERRNO_MBXLENGTH] =		US"MBX length mismatch",
+  [- ERRNO_UNKNOWNHOST] =	US"Lookup failed routing or in smtp tpt",
+  [- ERRNO_FORMATUNKNOWN] =	US"Can't match format in appendfile",
+  [- ERRNO_BADCREATE] =		US"Creation outside home in appendfile",
+  [- ERRNO_LISTDEFER] =		US"Can't check a list; lookup defer",
+  [- ERRNO_DNSDEFER] =		US"DNS lookup defer",
+  [- ERRNO_TLSFAILURE] =	US"Failed to start TLS session",
+  [- ERRNO_TLSREQUIRED] =	US"Mandatory TLS session not started",
+  [- ERRNO_CHOWNFAIL] =		US"Failed to chown a file",
+  [- ERRNO_PIPEFAIL] =		US"Failed to create a pipe",
+  [- ERRNO_CALLOUTDEFER] =	US"When verifying",
+  [- ERRNO_AUTHFAIL] =		US"When required by client",
+  [- ERRNO_CONNECTTIMEOUT] =	US"Used internally in smtp transport",
+  [- ERRNO_RCPT4XX] =		US"RCPT gave 4xx error",
+  [- ERRNO_MAIL4XX] =		US"MAIL gave 4xx error",
+  [- ERRNO_DATA4XX] =		US"DATA gave 4xx error",
+  [- ERRNO_PROXYFAIL] =		US"Negotiation failed for proxy configured host",
+  [- ERRNO_AUTHPROB] =		US"Authenticator 'other' failure",
+  [- ERRNO_UTF8_FWD] =		US"target not supporting SMTPUTF8",
+  [- ERRNO_HOST_IS_LOCAL] =	US"host is local",
+  [- ERRNO_TAINT] =		US"tainted filename",
 
-  US"Not time for routing",
-  US"Not time for local delivery",
-  US"Not time for any remote host",
-  US"Local-only delivery",
-  US"Domain in queue_domains",
-  US"Transport concurrency limit",
-  US"Event requests alternate response",
+  [- ERRNO_RRETRY] =		US"Not time for routing",
+
+  [- ERRNO_LRETRY] =		US"Not time for local delivery",
+  [- ERRNO_HRETRY] =		US"Not time for any remote host",
+  [- ERRNO_LOCAL_ONLY] =	US"Local-only delivery",
+  [- ERRNO_QUEUE_DOMAIN] =	US"Domain in queue_domains",
+  [- ERRNO_TRETRY] =		US"Transport concurrency limit",
+
+  [- ERRNO_EVENT] =		US"Event requests alternate response",
 };
 
 
@@ -266,7 +266,7 @@ Returns:       a file descriptor, or < 0 on failure (errno set)
 */
 
 static int
-log_open_already_exim(uschar * const name)
+log_open_already_exim(const uschar * const name)
 {
 int fd = -1;
 const int flags = O_WRONLY | O_APPEND | O_CREAT | O_NONBLOCK;
@@ -390,7 +390,7 @@ Returns:       a file descriptor, or < 0 on failure (errno set)
 */
 
 int
-log_open_as_exim(uschar * const name)
+log_open_as_exim(const uschar * const name)
 {
 int fd = -1;
 const uid_t euid = geteuid();
@@ -473,7 +473,7 @@ Returns:   nothing
 */
 
 static void
-open_log(int *fd, int type, uschar *tag)
+open_log(int * fd, int type, const uschar * tag)
 {
 uid_t euid;
 BOOL ok, ok2;
@@ -497,7 +497,6 @@ switch (type)
     it gets statted to see if it has been cycled. With a datestamp, the datestamp
     will be compared. The static slot for saving it is the same size as buffer,
     and the text has been checked above to fit, so this use of strcpy() is OK. */
-
     Ustrcpy(mainlog_name, buffer);
     if (string_datestamp_offset > 0)
       mainlog_datestamp = mainlog_name + string_datestamp_offset;
@@ -505,7 +504,6 @@ switch (type)
 
   case lt_reject:
     /* Ditto for the reject log */
-
     Ustrcpy(rejectlog_name, buffer);
     if (string_datestamp_offset > 0)
       rejectlog_datestamp = rejectlog_name + string_datestamp_offset;
@@ -514,13 +512,12 @@ switch (type)
   case lt_debug:
     /* and deal with the debug log (which keeps the datestamp, but does not
     update it) */
-
     Ustrcpy(debuglog_name, buffer);
     if (tag)
       {
       if (is_tainted(tag))
-	die(US"exim: tainted tag for debug log filename",
-	      US"Logging failure; please try later");
+      	die(US"exim: tainted tag for debug log filename",
+      	      US"Logging failure; please try later");
 
       /* this won't change the offset of the datestamp */
       ok2 = string_format(buffer, sizeof(buffer), "%s%s",
@@ -532,9 +529,8 @@ switch (type)
 
   default:
     /* Remove any datestamp if this is the panic log. This is rare, so there's no
-  need to optimize getting the datestamp length. We remove one non-alphanumeric
-  char afterwards if at the start, otherwise one before. */
-
+    need to optimize getting the datestamp length. We remove one non-alphanumeric
+    char afterwards if at the start, otherwise one before. */
     if (string_datestamp_offset >= 0)
       {
       uschar * from = buffer + string_datestamp_offset;
@@ -715,60 +711,31 @@ return total_written;
 }
 
 
-/* Pull the file out of the configured or the compiled-in list.
-Called for an empty log_file_path element, for debug logging activation
-when file_path has not previously been set, and from the appenfile transport setup. */
 
-void
-set_file_path(BOOL *multiple)
+static void
+set_file_path(void)
 {
-uschar *s;
 int sep = ':';              /* Fixed separator - outside use */
-const uschar *ss = *log_file_path ? log_file_path : US LOG_FILE_PATH;
-
-if (*ss)
-  for (logging_mode = 0;
-       s = string_nextinlist(&ss, &sep, log_buffer, LOG_BUFFER_SIZE); )
-    {
-    if (Ustrcmp(s, "syslog") == 0)
-      logging_mode |= LOG_MODE_SYSLOG;
-    else if (!(logging_mode & LOG_MODE_FILE))  /* no file yet */
-      {
-      logging_mode |= LOG_MODE_FILE;
-      if (*s) file_path = string_copy(s);     /* If a non-empty path is given, use it */
-      }
-    else if (multiple) *multiple = TRUE;
-    }
-else
-  logging_mode = LOG_MODE_FILE;
-
-/* Set up the ultimate default if necessary. */
-
-if (logging_mode & LOG_MODE_FILE  &&  !*file_path)
-  if (LOG_FILE_PATH[0])
-    {
-      /* If we still do not have a file_path, we take
-      the first non-empty, non-syslog item in LOG_FILE_PATH, if there is
-      one.  If there is no such item, use the ultimate default in the
-      spool directory. */
-
-       for (ss = US LOG_FILE_PATH;
-            s = string_nextinlist(&ss, &sep, log_buffer, LOG_BUFFER_SIZE);)
-          {
-            if (*s != '/') continue;
-            file_path = string_copy(s);
-          }
-    }
-  else file_path = string_sprintf("%s/log/%%slog", spool_directory);
+uschar *t;
+const uschar *tt = US LOG_FILE_PATH;
+while ((t = string_nextinlist(&tt, &sep, log_buffer, LOG_BUFFER_SIZE)))
+  {
+  if (Ustrcmp(t, "syslog") == 0 || t[0] == 0) continue;
+  file_path = string_copy(t);
+  break;
+  }
 }
 
 
+/* Close mainlog, unless we do not see a chance to open the file mainlog later
+again.  This will happen if we log from a transport process (which has dropped
+privs); something we traditionally avoid, but the introduction of taint-tracking
+and resulting detection of errors is makinng harder. */
+
 void
 mainlog_close(void)
 {
-/* avoid closing it if it is closed already or if we do not see a chance
-to open the file mainlog later again */
-if (mainlogfd < 0 /* already closed */
+if (mainlogfd < 0
    || !(geteuid() == 0 || geteuid() == exim_uid))
   return;
 (void)close(mainlogfd);
@@ -882,9 +849,41 @@ if (!path_inspected)
 
   store_pool = POOL_PERM;
 
-  /* make sure that we have a valid log file path in "file_path",
-  the open_log() later relies on it */
-  set_file_path(&multiple);
+  /* If nothing has been set, don't waste effort... the default values for the
+  statics are file_path="" and logging_mode = LOG_MODE_FILE. */
+
+  if (*log_file_path)
+    {
+    int sep = ':';              /* Fixed separator - outside use */
+    uschar *s;
+    const uschar *ss = log_file_path;
+
+    logging_mode = 0;
+    while ((s = string_nextinlist(&ss, &sep, log_buffer, LOG_BUFFER_SIZE)))
+      {
+      if (Ustrcmp(s, "syslog") == 0)
+        logging_mode |= LOG_MODE_SYSLOG;
+      else if (logging_mode & LOG_MODE_FILE)
+	multiple = TRUE;
+      else
+        {
+        logging_mode |= LOG_MODE_FILE;
+
+        /* If a non-empty path is given, use it */
+
+        if (*s)
+          file_path = string_copy(s);
+
+        /* If the path is empty, we want to use the first non-empty, non-
+        syslog item in LOG_FILE_PATH, if there is one, since the value of
+        log_file_path may have been set at runtime. If there is no such item,
+        use the ultimate default in the spool directory. */
+
+        else
+	  set_file_path();  /* Empty item in log_file_path */
+        }    /* First non-syslog item in log_file_path */
+      }      /* Scan of log_file_path */
+    }
 
   /* If no modes have been selected, it is a major disaster */
 
@@ -892,8 +891,11 @@ if (!path_inspected)
     die(US"Neither syslog nor file logging set in log_file_path",
         US"Unexpected logging failure");
 
-  /* Revert to the old store pool, and record that we've sorted out the path. */
+  /* Set up the ultimate default if necessary. Then revert to the old store
+  pool, and record that we've sorted out the path. */
 
+  if (logging_mode & LOG_MODE_FILE  &&  !file_path[0])
+    file_path = string_sprintf("%s/log/%%slog", spool_directory);
   store_pool = old_pool;
   path_inspected = TRUE;
 
@@ -905,6 +907,11 @@ if (!path_inspected)
       "More than one path given in log_file_path: using %s", file_path);
   }
 
+/* Optionally trigger debug */
+
+if (flags & LOG_PANIC && dtrigger_selector & BIT(DTi_panictrigger))
+  debug_trigger_fire();
+
 /* If debugging, show all log entries, but don't show headers. Do it all
 in one go so that it doesn't get split when multi-processing. */
 
@@ -1247,7 +1254,6 @@ if (flags & LOG_PANIC)
 
   if (logging_mode & LOG_MODE_FILE)
     {
-    if (!*file_path) set_file_path(NULL);
     panic_recurseflag = TRUE;
     open_log(&paniclogfd, lt_panic, NULL);  /* Won't return on failure */
     panic_recurseflag = FALSE;
@@ -1357,8 +1363,9 @@ Returns:         nothing on success - bomb out on failure
 */
 
 void
-decode_bits(unsigned int *selector, size_t selsize, int *notall,
-  uschar *string, bit_table *options, int count, uschar *which, int flags)
+decode_bits(unsigned int * selector, size_t selsize, int * notall,
+  const uschar * string, bit_table * options, int count, uschar * which,
+  int flags)
 {
 uschar *errmsg;
 if (!string) return;
@@ -1367,7 +1374,7 @@ if (*string == '=')
   {
   char *end;    /* Not uschar */
   memset(selector, 0, sizeof(*selector)*selsize);
-  *selector = strtoul(CS string+1, &end, 0);
+  *selector = strtoul(CCS string+1, &end, 0);
   if (!*end) return;
   errmsg = string_sprintf("malformed numeric %s_selector setting: %s", which,
     string);
@@ -1379,9 +1386,9 @@ if (*string == '=')
 else for(;;)
   {
   BOOL adding;
-  uschar *s;
+  const uschar * s;
   int len;
-  bit_table *start, *end;
+  bit_table * start, * end;
 
   Uskip_whitespace(&string);
   if (!*string) return;
@@ -1470,13 +1477,15 @@ misconfiguration.
 
 The first use of this is in ACL logic, "control = debug/tag=foo/opts=+expand"
 which can be combined with conditions, etc, to activate extra logging only
-for certain sources. The second use is inetd wait mode debug preservation. */
+for certain sources. The second use is inetd wait mode debug preservation.
+
+It might be nice, in ACL-initiated pretrigger mode, to not create the file
+immediately but only upon a trigger - but we'd need another cmdline option
+to pass the name through child_exxec_exim(). */
 
 void
-debug_logging_activate(uschar *tag_name, uschar *opts)
+debug_logging_activate(const uschar * tag_name, const uschar * opts)
 {
-int fd = -1;
-
 if (debug_file)
   {
   debug_printf("DEBUGGING ACTIVATED FROM WITHIN CONFIG.\n"
@@ -1484,7 +1493,7 @@ if (debug_file)
   return;
   }
 
-if (tag_name != NULL && (Ustrchr(tag_name, '/') != NULL))
+if (tag_name && (Ustrchr(tag_name, '/') != NULL))
   {
   log_write(0, LOG_MAIN|LOG_PANIC, "debug tag may not contain a '/' in: %s",
       tag_name);
@@ -1500,36 +1509,46 @@ if (opts)
 resulting in certain setup not having been done.  Hack this for now so we
 do not segfault; note that nondefault log locations will not work */
 
-if (!*file_path) set_file_path(NULL);
+if (!*file_path) set_file_path();
 
-open_log(&fd, lt_debug, tag_name);
+open_log(&debug_fd, lt_debug, tag_name);
 
-if (fd != -1)
-  debug_file = fdopen(fd, "w");
+if (debug_fd != -1)
+  debug_file = fdopen(debug_fd, "w");
 else
   log_write(0, LOG_MAIN|LOG_PANIC, "unable to open debug log");
 }
 
 
 void
-debug_logging_stop(void)
+debug_logging_from_spool(const uschar * filename)
 {
+if (debug_fd < 0)
+  {
+  Ustrncpy(debuglog_name, filename, sizeof(debuglog_name));
+  if ((debug_fd = log_open_as_exim(filename)) >= 0)
+    debug_file = fdopen(debug_fd, "w");
+  DEBUG(D_deliver) debug_printf("debug enabled by spoolfile\n");
+  }
+/*
+else DEBUG(D_deliver)
+  debug_printf("debug already active; ignoring spoolfile '%s'\n", filename);
+*/
+}
+
+
+void
+debug_logging_stop(BOOL kill)
+{
+debug_pretrigger_discard();
 if (!debug_file || !debuglog_name[0]) return;
 
 debug_selector = 0;
 fclose(debug_file);
 debug_file = NULL;
-unlink_log(lt_debug);
+debug_fd = -1;
+if (kill) unlink_log(lt_debug);
 }
 
-/* Called from the appendfile transport setup. */
-void
-open_logs(void)
-{
-set_file_path(NULL);
-if (!(logging_mode & LOG_MODE_FILE)) return;
-open_log(&mainlogfd, lt_main, 0);
-open_log(&rejectlogfd, lt_reject, 0);
-}
 
 /* End of log.c */
