@@ -5,14 +5,13 @@
 /* Exim - SPF lookup module using libspf2
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
+Copyright (c) The Exim Maintainers 2020 - 2022
 Copyright (c) 2005 Chris Webb, Arachsys Internet Services Ltd
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
-
-Copyright (c) The Exim Maintainers 2020
 */
 
 #include "../exim.h"
@@ -128,12 +127,13 @@ return OK;
 
 #include "../version.h"
 
-void
-spf_version_report(FILE *f)
+gstring *
+spf_version_report(gstring * g)
 {
 #ifdef DYNLOOKUP
-fprintf(f, "Library version: SPF: Exim version %s\n", EXIM_VERSION_STR);
+g = string_fmt_append(g, "Library version: SPF: Exim version %s\n", EXIM_VERSION_STR));
 #endif
+return g;
 }
 
 
