@@ -2,8 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2020 - 2022 */
 /* Copyright (c) University of Cambridge 1995 - 2015 */
-/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This code originally came from Robert Wal. */
@@ -23,8 +23,6 @@
 static void *
 whoson_open(const uschar * filename, uschar ** errmsg)
 {
-filename = filename;   /* Keep picky compilers happy */
-errmsg = errmsg;
 return (void *)(1);    /* Just return something non-null */
 }
 
@@ -40,11 +38,6 @@ whoson_find(void * handle, const uschar * filename, uschar * query, int length,
   uschar ** result, uschar ** errmsg, uint * do_cache, const uschar * opts)
 {
 uschar buffer[80];
-handle = handle;          /* Keep picky compilers happy */
-filename = filename;
-length = length;
-errmsg = errmsg;
-do_cache = do_cache;
 
 switch (wso_query(CS query, CS buffer, sizeof(buffer)))
   {
@@ -71,13 +64,16 @@ switch (wso_query(CS query, CS buffer, sizeof(buffer)))
 
 #include "../version.h"
 
-void
-whoson_version_report(FILE *f)
+gstring *
+whoson_version_report(gstring * g)
 {
-fprintf(f, "Library version: Whoson: Runtime: %s\n", wso_version());
+g = string_fmt_append(g,
+  "Library version: Whoson: Runtime: %s\n", wso_version());
 #ifdef DYNLOOKUP
-fprintf(f, "                         Exim version %s\n", EXIM_VERSION_STR);
+g = string_fmt_append(g,
+  "                         Exim version %s\n", EXIM_VERSION_STR);
 #endif
+return g;
 }
 
 static lookup_info _lookup_info = {
