@@ -2,8 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2020 - 2022 */
 /* Copyright (c) Jeremy Harris 1995 - 2018 */
-/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Create a static data structure with the predefined macros, to be
@@ -183,7 +183,7 @@ due to conflicts with other common macros. */
 #ifdef SUPPORT_SPF
   builtin_macro_create(US"_HAVE_SPF");
 #endif
-#if defined(EXPERIMENTAL_SRS_ALT) || defined(SUPPORT_SRS)
+#ifdef SUPPORT_SRS
   builtin_macro_create(US"_HAVE_SRS");
 #endif
 #ifdef EXPERIMENTAL_ARC
@@ -233,9 +233,6 @@ due to conflicts with other common macros. */
 #ifdef LOOKUP_LDAP
   builtin_macro_create(US"_HAVE_LOOKUP_LDAP");
 #endif
-#ifdef EXPERIMENTAL_LMDB
-  builtin_macro_create(US"_HAVE_LOOKUP_LMDB");
-#endif
 #ifdef LOOKUP_MYSQL
   builtin_macro_create(US"_HAVE_LOOKUP_MYSQL");
 #endif
@@ -308,9 +305,6 @@ exp_features(void)
 #ifdef EXPERIMENTAL_QUEUEFILE
   builtin_macro_create(US"_EXP_QUEUEFILE");
 #endif
-#if defined(EXPERIMENTAL_SRS_ALT)
-  builtin_macro_create(US"_EXP_SRS");
-#endif
 }
 
 
