@@ -2,8 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2020 - 2022 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
-/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions that operate on the input queue. */
@@ -214,8 +214,8 @@ for (; i <= *subcount; i++)
 	(*pcount)++;
       else
 	{
-	queue_filename *next =
-	  store_get(sizeof(queue_filename) + Ustrlen(name), is_tainted(name));
+	queue_filename * next =
+	  store_get(sizeof(queue_filename) + Ustrlen(name), name);
 	Ustrcpy(next->text, name);
 	next->dir_uschar = subdirchar;
 
@@ -350,8 +350,8 @@ queue_run(uschar *start_id, uschar *stop_id, BOOL recurse)
 {
 BOOL force_delivery = f.queue_run_force || deliver_selectstring != NULL ||
   deliver_selectstring_sender != NULL;
-const pcre *selectstring_regex = NULL;
-const pcre *selectstring_regex_sender = NULL;
+const pcre2_code *selectstring_regex = NULL;
+const pcre2_code *selectstring_regex_sender = NULL;
 uschar *log_detail = NULL;
 int subcount = 0;
 uschar subdirs[64];
@@ -569,9 +569,7 @@ for (int i = queue_run_in_order ? -1 : 0;
 
       else if (  deliver_selectstring_sender
 	      && !(f.deliver_selectstring_sender_regex
-		  ? (pcre_exec(selectstring_regex_sender, NULL,
-		      CS sender_address, Ustrlen(sender_address), 0, PCRE_EOPT,
-		      NULL, 0) >= 0)
+		  ? regex_match(selectstring_regex_sender, sender_address, -1, NULL)
 		  : (strstric(sender_address, deliver_selectstring_sender, FALSE)
 		      != NULL)
 	      )   )
@@ -590,8 +588,7 @@ for (int i = queue_run_in_order ? -1 : 0;
           {
           uschar *address = recipients_list[i].address;
           if (  (f.deliver_selectstring_regex
-		? (pcre_exec(selectstring_regex, NULL, CS address,
-		     Ustrlen(address), 0, PCRE_EOPT, NULL, 0) >= 0)
+		? regex_match(selectstring_regex, address, -1, NULL)
                 : (strstric(address, deliver_selectstring, FALSE) != NULL)
 		)
              && tree_search(tree_nonrecipients, address) == NULL
@@ -904,8 +901,8 @@ if (count > 0)
   queue_filename *last = NULL;
   for (int i = 0; i < count; i++)
     {
-    queue_filename *next =
-      store_get(sizeof(queue_filename) + Ustrlen(list[i]) + 2, is_tainted(list[i]));
+    queue_filename * next =
+      store_get(sizeof(queue_filename) + Ustrlen(list[i]) + 2, list[i]);
     sprintf(CS next->text, "%s-H", list[i]);
     next->dir_uschar = '*';
     next->next = NULL;
@@ -1351,15 +1348,15 @@ switch(action)
 	    deliver_domain = dom
 	      ? CUS string_copyn(addr+dom, end - dom) : CUS"";
 
-	    event_raise(event_action, US"msg:fail:internal",
-	      string_sprintf("message removed by %s", username));
+	    (void) event_raise(event_action, US"msg:fail:internal",
+	      string_sprintf("message removed by %s", username), NULL);
 
 	    deliver_localpart = save_local;
 	    deliver_domain = save_domain;
 	    }
 	  }
 	}
-      (void) event_raise(event_action, US"msg:complete", NULL);
+      (void) event_raise(event_action, US"msg:complete", NULL, NULL);
 #endif
       log_write(0, LOG_MAIN, "removed by %s", username);
       log_write(0, LOG_MAIN, "Completed");
