@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -18,6 +18,8 @@ typedef struct {
   uschar *srv_fail_domains;
   uschar *check_srv;
   uschar *fail_defer_domains;
+  uschar *ipv4_only;
+  uschar *ipv4_prefer;
 } dnslookup_router_options_block;
 
 /* Data for reading the private options. */
